/***************************************************************************//**
* @file     A31R71x_hal_adc.h
* @brief    Contains all macro definitions and function prototypes
*           support for adc firmware library on A31R71x
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements,
*modifications, and improvements to ABOV products and/or to this document at any time without notice.
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders.
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and
*ABOV assumes no liability for application assistance or the design of purchasers' products. No license,
*express or implied, to any intellectual property rights is granted by ABOV herein.
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications.
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners.
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
*//****************************************************************************/

#ifndef _ADC_H_
#define _ADC_H_

#include "A31R71x.h"
#include "A31R71x_hal_aa_types.h"

#ifdef __cplusplus
extern "C" {
#endif

//******************************************************************************
// Constant
//******************************************************************************

// ADC Trigger Signal Selection Control
/*
#define ADC_ADST                    (0x0uL << 11)
#define ADC_T10M                    (0x1uL << 11)
#define ADC_T11M                    (0x2uL << 11)
#define ADC_T12M                    (0x3uL << 11)
#define ADC_T30                     (0x4uL << 11)
*/
#define TRIG_ADST                   (ADC_CR_TRIG_ADST    << ADC_CR_TRIG_Pos)      // ADST : Select ADST
#define TRIG_TIMER10                (ADC_CR_TRIG_TIMER10 << ADC_CR_TRIG_Pos)      // TIMER10 : Select Timer 10 A-Match Signal
#define TRIG_TIMER11                (ADC_CR_TRIG_TIMER11 << ADC_CR_TRIG_Pos)      // TIMER11 : Select Timer 11 A-Match Signal
#define TRIG_TIMER12                (ADC_CR_TRIG_TIMER12 << ADC_CR_TRIG_Pos)      // TIMER12 : Select Timer 12 A-Match Signal

// ADC Reference Selection Control
/*
#define ADC_VDD                     (0x0uL << 10)
#define ADC_AVREF                   (0x1uL << 10)
*/
#define REFSEL_Vdd                  (ADC_CR_REFSEL_Vdd   << ADC_CR_REFSEL_Pos)      // Vdd : Select analog power (VDD)
#define REFSEL_AVref                (ADC_CR_REFSEL_AVref << ADC_CR_REFSEL_Pos)      // AVref : Select external reference (AVREF)

// ADC Interrupt Enable Control
/*
#define ADC_INTDIS                  (0x0uL << 5)
#define ADC_INTEN                   (0x1uL << 5)
*/
#define ADCIEN_Disable              (ADC_CR_ADCIEN_Disable << ADC_CR_ADCIEN_Pos)    // Disable : Disable ADC interrupt
#define ADCIEN_Enable               (ADC_CR_ADCIEN_Enable  << ADC_CR_ADCIEN_Pos)    // Enable : Enable ADC interrupt

// A/D Converter Channel Selection Control
/*
#define ADC_AN0                     (0x00uL << 0)
#define ADC_AN1                     (0x01uL << 0)
#define ADC_AN2                     (0x02uL << 0)
#define ADC_AN3                     (0x03uL << 0)
#define ADC_AN4                     (0x04uL << 0)
#define ADC_AN5                     (0x05uL << 0)
#define ADC_AN6                     (0x06uL << 0)
#define ADC_AN7                     (0x07uL << 0)
#define ADC_AN8                     (0x08uL << 0)
#define ADC_AN9                     (0x09uL << 0)
#define ADC_AN10                    (0x0auL << 0)
*/
#define ADSEL_AN0                   (ADC_CR_ADSEL_AN0  << ADC_CR_ADSEL_Pos)      // AN0 : Select AN0
#define ADSEL_AN1                   (ADC_CR_ADSEL_AN1  << ADC_CR_ADSEL_Pos)      // AN1 : Select AN1
#define ADSEL_AN2                   (ADC_CR_ADSEL_AN2  << ADC_CR_ADSEL_Pos)      // AN2 : Select AN2
#define ADSEL_AN3                   (ADC_CR_ADSEL_AN3  << ADC_CR_ADSEL_Pos)      // AN3 : Select AN3
#define ADSEL_AN4                   (ADC_CR_ADSEL_AN4  << ADC_CR_ADSEL_Pos)      // AN4 : Select AN4
#define ADSEL_AN5                   (ADC_CR_ADSEL_AN5  << ADC_CR_ADSEL_Pos)      // AN5 : Select AN5
#define ADSEL_AN6                   (ADC_CR_ADSEL_AN6  << ADC_CR_ADSEL_Pos)      // AN6 : Select AN6
#define ADSEL_AN7                   (ADC_CR_ADSEL_AN7  << ADC_CR_ADSEL_Pos)      // AN7 : Select AN7
#define ADSEL_AN8                   (ADC_CR_ADSEL_AN8  << ADC_CR_ADSEL_Pos)      // AN8 : Select AN8
#define ADSEL_AN9                   (ADC_CR_ADSEL_AN9  << ADC_CR_ADSEL_Pos)      // AN9 : Select AN9
#define ADSEL_AN10                  (ADC_CR_ADSEL_AN10 << ADC_CR_ADSEL_Pos)      // AN10 : Select AN10
#define ADSEL_AN11                  (ADC_CR_ADSEL_AN11 << ADC_CR_ADSEL_Pos)      // AN11 : Select AN11
#define ADSEL_AN12                  (ADC_CR_ADSEL_AN12 << ADC_CR_ADSEL_Pos)      // AN12 : Select AN12
#define ADSEL_AN13                  (ADC_CR_ADSEL_AN13 << ADC_CR_ADSEL_Pos)      // AN13 : Select AN13

//******************************************************************************
// Type
//******************************************************************************

//==============================================================================
// Structure
//==============================================================================

typedef struct
{
   /* ADC_CR */
   uint32_t    RefSel;           /**<  RestartEn = ENABLE or DISABLE */
   uint32_t    TrgSel;

   /* ADC_PREDR */
   uint32_t    InClkDiv;
} ADC_CFG_Type;

//******************************************************************************
// Macro
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         ADC Enable
 * @details       This macro Enable ADC Block
 *//*-------------------------------------------------------------------------*/
#define ADCEN()                     (ADC->CR_b.ADCEN = 1)

/*-------------------------------------------------------------------------*//**
 * @brief         ADC Disable
 * @details       This macro Disable ADC Block
 *//*-------------------------------------------------------------------------*/
#define ADCDIS()                    (ADC->CR_b.ADCEN = 0)

/*-------------------------------------------------------------------------*//**
 * @brief         ADC Start bit set
 * @details       This macro starts ADC conversion
 *//*-------------------------------------------------------------------------*/
#define ADCADST_Set()               (ADC->CR_b.ADST = 1)

/*-------------------------------------------------------------------------*//**
 * @brief         ADC Start bit get
 * @details       This macro gets ADST bit
 *//*-------------------------------------------------------------------------*/
#define ADCADST_Get()               (ADC->CR_b.ADST)

/*-------------------------------------------------------------------------*//**
 * @brief         Get flags of ADC Interrupt
 * @details       This macro gets interrupt flag of ADC
 *//*-------------------------------------------------------------------------*/
#define ADCInt_GetFg()              (ADC->CR_b.ADCIFLAG)

/*-------------------------------------------------------------------------*//**
 * @brief         Clear flags of ADC Interrupt
 * @details       This macro gets interrupt flag of ADC
 *//*-------------------------------------------------------------------------*/
#define ADCInt_ClrFg()              (ADC->CR_b.ADCIFLAG = 1)

/*-------------------------------------------------------------------------*//**
 * @brief         Get data of ADC Conversion
 * @details       This macro gets data of ADC Conversion
 *//*-------------------------------------------------------------------------*/
#define ADCData_Get()               (ADC->DR)

/*-------------------------------------------------------------------------*//**
 * Macro defines for ADC Data register
 *//*-------------------------------------------------------------------------*/
/**   When DONE is 1, this field contains result value of ADC conversion */
#define ADC_DR_RESULT( n )          ((n) & ADC_DR_ADDATA_Msk)

//******************************************************************************
// Function
//******************************************************************************

HAL_Status_Type HAL_ADC_Init( ADC_Type* ADCx, ADC_CFG_Type* ADC_Config );
HAL_Status_Type HAL_ADC_DeInit( ADC_Type* ADCx );

HAL_Status_Type HAL_ADC_ConfigInterrupt( ADC_Type* ADCx, FunctionalState NewState );
HAL_Status_Type HAL_ADC_ChannelSel( ADC_Type* ADCx, uint32_t Channel );
HAL_Status_Type HAL_ADC_Start( ADC_Type* ADCx );
HAL_Status_Type HAL_ADC_Stop( ADC_Type* ADCx );
HAL_Status_Type HAL_ADC_ClearStatus( ADC_Type* ADCx );
uint32_t HAL_ADC_GetStatus( ADC_Type* ADCx );

uint16_t HAL_ADC_GetData( ADC_Type* ADCx );

#ifdef __cplusplus
}
#endif

#endif   /* _ADC_H_ */

