/***************************************************************************//**
* @file     A31R71x_hal_lcd.h
* @brief    Contains all macro definitions and function prototypes
*           support for lcd firmware library on A31R71x
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements,
*modifications, and improvements to ABOV products and/or to this document at any time without notice.
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders.
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and
*ABOV assumes no liability for application assistance or the design of purchasers' products. No license,
*express or implied, to any intellectual property rights is granted by ABOV herein.
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications.
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners.
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
*//****************************************************************************/

#ifndef _LCD_H_
#define _LCD_H_

#include "A31R71x.h"
#include "A31R71x_hal_aa_types.h"

#ifdef __cplusplus
extern "C" {
#endif

//******************************************************************************
// Constant
//******************************************************************************

// LCD Bias Type Selection Control
#define BTYPE_IntRBias  (LCD_CR_BTYPE_IntRBias << LCD_CR_BTYPE_Pos)  // IntRBias : Internal Resistor Bias
#define BTYPE_ExtRBias  (LCD_CR_BTYPE_ExtRBias << LCD_CR_BTYPE_Pos)  // ExtRBias : External Resistor Bias
#define BTYPE_CapBias   (LCD_CR_BTYPE_CapBias  << LCD_CR_BTYPE_Pos)  // CapBias : Capacitor Bias (Voltage Booster)

// Internal LCD Bias Dividing Resistor Selection Control
/*
#define LCD_RLCD3       (0x0uL << 6)
#define LCD_RLCD1       (0x1uL << 6)
#define LCD_RLCD2       (0x2uL << 6)
#define LCD_RLCD4       (0x3uL << 6)
*/
#define IRSEL_RLCD2     (LCD_CR_IRSEL_RLCD2 << LCD_CR_IRSEL_Pos)  // RLCD2 : 60[kohm] 
#define IRSEL_RLCD1     (LCD_CR_IRSEL_RLCD1 << LCD_CR_IRSEL_Pos)  // RLCD1 : 30[kohm] 
#define IRSEL_RLCD3     (LCD_CR_IRSEL_RLCD3 << LCD_CR_IRSEL_Pos)  // RLCD3 : 120[kohm]

// LCD Duty and Bias Selection Control
/*
#define LCD_8D4B        (0x0uL << 3)
#define LCD_6D4B        (0x1uL << 3)
#define LCD_5D3B        (0x2uL << 3)
#define LCD_4D3B        (0x3uL << 3)
#define LCD_3D3B        (0x4uL << 3)
#define LCD_3D2B        (0x5uL << 3)
*/
#define DBS_4D3B        (LCD_CR_DBS_4D3B << LCD_CR_DBS_Pos)    // 4D3B : 1/4 duty, 1/3 bias
#define DBS_3D3B        (LCD_CR_DBS_3D3B << LCD_CR_DBS_Pos)    // 3D3B : 1/3 duty, 1/3 bias
#define DBS_3D2B        (LCD_CR_DBS_3D2B << LCD_CR_DBS_Pos)    // 3D2B : 1/3 duty, 1/2 bias
#define DBS_2D2B        (LCD_CR_DBS_2D2B << LCD_CR_DBS_Pos)    // 2D2B : 1/2 duty, 1/2 bias

// LCD Clock Selection Control
/*
#define LCD_CLK128      (0x0uL << 1)
#define LCD_CLK256      (0x1uL << 1)
#define LCD_CLK512      (0x2uL << 1)
#define LCD_CLK1024     (0x3uL << 1)
*/
#define LCLK_128Hz      (LCD_CR_LCLK_128Hz  << LCD_CR_LCLK_Pos)   // 128Hz : 128Hz = 32.768kHz(fLCD) / 256
#define LCLK_256Hz      (LCD_CR_LCLK_256Hz  << LCD_CR_LCLK_Pos)   // 256Hz : 256Hz = 32.768kHz(fLCD) / 128
#define LCLK_512Hz      (LCD_CR_LCLK_512Hz  << LCD_CR_LCLK_Pos)   // 512Hz : 512Hz = 32.768kHz(fLCD) / 64
#define LCLK_1024Hz     (LCD_CR_LCLK_1024Hz << LCD_CR_LCLK_Pos)   // 1024Hz : 1024Hz = 32.768kHz(fLCD) / 32

// VLC2 Voltage Control
/*
#define LCD_VLC31       (0x0uL << 0)
#define LCD_VLC30       (0x1uL << 0)
#define LCD_VLC29       (0x2uL << 0)
#define LCD_VLC28       (0x3uL << 0)
#define LCD_VLC27       (0x4uL << 0)
#define LCD_VLC26       (0x5uL << 0)
#define LCD_VLC25       (0x6uL << 0)
#define LCD_VLC24       (0x7uL << 0)
#define LCD_VLC23       (0x8uL << 0)
#define LCD_VLC22       (0x9uL << 0)
#define LCD_VLC21       (0xAuL << 0)
#define LCD_VLC20       (0xBuL << 0)
#define LCD_VLC19       (0xCuL << 0)
#define LCD_VLC18       (0xDuL << 0)
#define LCD_VLC17       (0xEuL << 0)
#define LCD_VLC16       (0xFuL << 0)
*/
#define VLCD_Step0      (LCD_CCR_VLCD_Step0  << LCD_CCR_VLCD_Pos)    // Step0 : VLC2=1.00V when 1/2 Bias, VLC2=0.75V when 1/3 Bias
#define VLCD_Step1      (LCD_CCR_VLCD_Step1  << LCD_CCR_VLCD_Pos)    // Step1 : VLC2=1.05V when 1/2 Bias, VLC2=0.79V when 1/3 Bias
#define VLCD_Step2      (LCD_CCR_VLCD_Step2  << LCD_CCR_VLCD_Pos)    // Step2 : VLC2=1.10V when 1/2 Bias, VLC2=0.83V when 1/3 Bias
#define VLCD_Step3      (LCD_CCR_VLCD_Step3  << LCD_CCR_VLCD_Pos)    // Step3 : VLC2=1.15V when 1/2 Bias, VLC2=0.86V when 1/3 Bias
#define VLCD_Step4      (LCD_CCR_VLCD_Step4  << LCD_CCR_VLCD_Pos)    // Step4 : VLC2=1.20V when 1/2 Bias, VLC2=0.90V when 1/3 Bias
#define VLCD_Step5      (LCD_CCR_VLCD_Step5  << LCD_CCR_VLCD_Pos)    // Step5 : VLC2=1.25V when 1/2 Bias, VLC2=0.94V when 1/3 Bias
#define VLCD_Step6      (LCD_CCR_VLCD_Step6  << LCD_CCR_VLCD_Pos)    // Step6 : VLC2=1.30V when 1/2 Bias, VLC2=0.98V when 1/3 Bias
#define VLCD_Step7      (LCD_CCR_VLCD_Step7  << LCD_CCR_VLCD_Pos)    // Step7 : VLC2=1.35V when 1/2 Bias, VLC2=1.01V when 1/3 Bias
#define VLCD_Step8      (LCD_CCR_VLCD_Step8  << LCD_CCR_VLCD_Pos)    // Step8 : VLC2=1.40V when 1/2 Bias, VLC2=1.05V when 1/3 Bias
#define VLCD_Step9      (LCD_CCR_VLCD_Step9  << LCD_CCR_VLCD_Pos)    // Step9 : VLC2=1.45V when 1/2 Bias, VLC2=1.09V when 1/3 Bias
#define VLCD_Step10     (LCD_CCR_VLCD_Step10 << LCD_CCR_VLCD_Pos)    // Step10 : VLC2=1.50V when 1/2 Bias, VLC2=1.13V when 1/3 Bias


// Display Data RAM Size
#define LCDBufSize      40

//******************************************************************************
// Type
//******************************************************************************

//==============================================================================
// Structure
//==============================================================================

typedef struct
{
   // LCD_CR
   uint32_t    BiasType;      // BTYPE
   uint32_t    Bias;          // IRSEL
   uint32_t    Duty;          // DBS
   uint32_t    Clk;           // LCLK

   // LCD_CCR
   uint32_t    ContrastStep;  // VLCD
} LCD_CFG_Type;

//******************************************************************************
// Macro
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         LCD Enable
 * @details       This macro Enable LCD Block
 *//*-------------------------------------------------------------------------*/
#define LCDON()         (LCD->CR_b.DISP = 1)

/*-------------------------------------------------------------------------*//**
 * @brief         LCD Disable
 * @details       This macro Disable LCD Block
 *//*-------------------------------------------------------------------------*/
#define LCDOFF()        (LCD->CR_b.DISP = 0)

//******************************************************************************
// Function
//******************************************************************************

HAL_Status_Type HAL_LCD_Init( LCD_CFG_Type* LCD_Config );

HAL_Status_Type HAL_LCD_SetRegister( uint32_t LCD_CR, uint32_t LCD_CCR );
HAL_Status_Type HAL_LCD_ClearDspRam( void );
HAL_Status_Type HAL_LCD_WriteDspRam( uint8_t* write_buf, uint32_t u32Index, uint32_t size );

#ifdef __cplusplus
}
#endif

#endif   /* _LCD_H_ */

