/***************************************************************************//**
* @file     A31R71x_hal_sculv.c
* @brief    Contains all functions support for sculv firmware library on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements,
*modifications, and improvements to ABOV products and/or to this document at any time without notice.
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders.
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and
*ABOV assumes no liability for application assistance or the design of purchasers' products. No license,
*express or implied, to any intellectual property rights is granted by ABOV herein.
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications.
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners.
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
*//****************************************************************************/

/* Includes ----------------------------------------------------------------- */
//******************************************************************************
// Include
//******************************************************************************

#include "A31R71x_hal_sculv.h"
#include "A31R71x_hal_scu.h"

/* Public Functions --------------------------------------------------------- */
//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         Set LVI Block Enable/Disable and Voltage Level
 * @param[in]     u32LviEnBit
 *                   LVI Enable/Disable Control
 *                   -  LVIEN_Disable, LVIEN_Enable
 * @param[in]     u32LviIntEnBit
 *                   LVI Interrupt Enable/Disable Control
 *                   -  LVINTEN_Disable, LVINTEN_Enable
 * @param[in]     u32LviLevel
 *                   LVI Voltage Level
 *                   -  LVIVS_2p00V    // 2.00V
 *                   -  LVIVS_2p13V    // 2.13V
 *                   -  LVIVS_2p28V    // 2.28V
 *                   -  LVIVS_2p46V    // 2.46V
 *                   -  LVIVS_2p67V    // 2.67V
 *                   -  LVIVS_3p04V    // 3.04V
 *                   -  LVIVS_3p20V    // 3.20V
 *                   -  LVIVS_3p55V    // 3.55V
 *                   -  LVIVS_3p75V    // 3.75V
 *                   -  LVIVS_3p99V    // 3.99V
 *                   -  LVIVS_4p25V    // 4.25V
 *                   -  LVIVS_4p55V    // 4.55V
 * @return        None
 * @details       This function sets LVI voltage level.
 *//*-------------------------------------------------------------------------*/
void HAL_LVI_Init( uint32_t u32LviEnBit, uint32_t u32LviIntEnBit, uint32_t u32LviLevel )
{
   // enable peripheral clock
   HAL_SCU_Peripheral_EnableClock2( PPCLKEN2_LVICLKE, PPxCLKE_Enable );

   SCULV->LVICR = ( u32LviEnBit | u32LviIntEnBit | u32LviLevel );
}

