/***************************************************************************//**
* @file     lcd_8D3B.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

//******************************************************************************
// Variable
//******************************************************************************

static uint8_t*         lcdfbuf;

static const uint8_t    xLCD_BUF[30] =
{
   0x7C, 0x12, 0x11, 0x12, 0x7C,    // "A"   0AH
   0x41, 0x7F, 0x49, 0x49, 0x36,    // "B"   0BH
   0x3E, 0x41, 0x41, 0x41, 0x22,    // "C"   0CH
   0x41, 0x7F, 0x41, 0x41, 0x3E,    // "D"   0DH
   0x7F, 0x49, 0x49, 0x49, 0x41,    // "E"   0EH
   0x7F, 0x09, 0x09, 0x09, 0x01,    // "F"   0FH
};

//******************************************************************************
// Function
//******************************************************************************

static void lcd_setting( void )
{
   char              i;
   LCD_CFG_Type      LCD_Config;

   // select peripheral clock: WDTRC
   HAL_SCU_Peripheral_ClockSelection( PPCLKSR_LCDCLK, LCDCLK_WDTRC );

   // init lcd
   LCD_Config.BiasType = BTYPE_IntRBias;     // BTYPE
   LCD_Config.Bias = IRSEL_RLCD2;            // IRSEL
   LCD_Config.Duty = DBS_4D3B;               // DBS
   LCD_Config.Clk = LCLK_512Hz;              // LCLK
   LCD_Config.ContrastStep = VLCD_Step0;     // VLCD
   HAL_LCD_Init( &LCD_Config );

   // start address of Display Data RAM
   lcdfbuf = ( uint8_t* )( 0x40005010 );     // &LCD->DR0

   // init Display Data RAM
   for( i = 0; i < LCDBufSize; i++ )
   {
      lcdfbuf[i] = 0x00;
   }

   // lcd bias voltage pin configuration
   {
      // external resistor bias voltage pin configuration
      if( LCD_Config.BiasType == BTYPE_ExtRBias )
      {
         // print
         _DBG( "PF5 ~ PF7 was configured as a VLC0 ~ VLC2. \n\r" );

         // configure PF5 ~ PF7 as a VLC0 ~ VLC2
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 7, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 7, 0 );        // PF7            0: VLC2        1: ----        2: ----        3: ----        4: ----
         if( LCD_Config.Duty == DBS_4D3B || LCD_Config.Duty == DBS_3D3B )
         {
            HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 6, ALTERN_FUNC );
            HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 6, 0 );     // PF6            0: VLC1        1: ----        2: ----        3: ----        4: ----
         }
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 5, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 5, 0 );        // PF5            0: VLC0        1: ----        2: ----        3: ----        4: ----
      }

      // capacitor bias voltage pin configuration
      if( LCD_Config.BiasType == BTYPE_CapBias )
      {
         // print
         _DBG( "PF5 ~ PF7 was configured as a VLC0 ~ VLC2. \n\r" );
         _DBG( "PE0 ~ PE1 was configured as a CAPH ~ CAPL. \n\r\n\r" );

         // configure PF5 ~ PF7 as a VLC0 ~ VLC2
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 7, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 7, 0 );        // PF7            0: VLC2        1: ----        2: ----        3: ----        4: ----
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 6, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 6, 0 );        // PF6            0: VLC1        1: ----        2: ----        3: ----        4: ----
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 5, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 5, 0 );        // PF5            0: VLC0        1: ----        2: ----        3: ----        4: ----

         // configure PE0 ~ PE1 as a CAPH ~ CAPL
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 1, 0 );        // PE1            0: CAPL        1: ----        2: ----        3: ----        4: ----
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 0, 0 );        // PE0            0: CAPH        1: ----        2: ----        3: ----        4: ----
      }
   }

   // lcd signal pin configuration
   {
      // print
      _DBG( "PE2 ~ PE3 was configured as a COM0 ~ COM1. \n\r" );
      _DBG( "PE4 ~ PE5 was configured as a COM2/SEG0 ~ COM3/SEG1. \n\r" );
      _DBG( "PE6 ~ PE7 was configured as a SEG2 ~ SEG3. \n\r" );
      _DBG( "PD0 ~ PD7 was configured as a SEG11 ~ SEG4. \n\r" );
      _DBG( "PC0 ~ PC11 was configured as a SEG23 ~ SEG12. \n\r" );
      _DBG( "PB8 ~ PB15 was configured as a SEG31 ~ SEG24. \n\r" );
      _DBG( "PB0 ~ PB7 was configured as a SEG39 ~ SEG32 (except for PB4 ~ PB5). \n\r" );

      // configure PE2 ~ PE3 as a COM0 ~ COM1
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 3, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 3, 0 );        // PE3            0: COM1        1: ----        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 2, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 2, 0 );        // PE2            0: COM0        1: ----        2: ----        3: ----        4: ----

      // configure PE4 ~ PE5 as a COM2/SEG0 ~ COM3/SEG1
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 5, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 5, 0 );        // PE5            0: COM3/SEG1   1: ----        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 4, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 4, 0 );        // PE4            0: COM2/SEG0   1: ----        2: ----        3: ----        4: ----

      // configure PE6 ~ PE7 as a SEG2 ~ SEG3
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 7, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 7, 0 );        // PE7            0: SEG3        1: T11OUT      2: T11CAP      3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 6, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 6, 0 );        // PE6            0: SEG2        1: T10OUT      2: T10CAP      3: ----        4: ----

      // configure PD0 ~ PD7 as a SEG11 ~ SEG4
      PD->MOD     =  0
                     | ( 2 << PD_PD_MOD_MODE7_Pos )         // PD7            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PD_PD_MOD_MODE6_Pos )         // PD6            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PD_PD_MOD_MODE5_Pos )         // PD5            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PD_PD_MOD_MODE4_Pos )         // PD4            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PD_PD_MOD_MODE3_Pos )         // PD3            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PD_PD_MOD_MODE2_Pos )         // PD2            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PD_PD_MOD_MODE1_Pos )         // PD1            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PD_PD_MOD_MODE0_Pos )         // PD0            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     ;
      PD->AFSR1   =  0
                     | ( 0 << PD_PD_AFSR1_AFSR7_Pos )       // PD7            0: SEG4        1: EC10        2: ----        3: ----        4: ----
                     | ( 0 << PD_PD_AFSR1_AFSR6_Pos )       // PD6            0: SEG5        1: EC11        2: ----        3: ----        4: ----
                     | ( 0 << PD_PD_AFSR1_AFSR5_Pos )       // PD5            0: SEG6        1: ----        2: SS11        3: ----        4: ----
                     | ( 0 << PD_PD_AFSR1_AFSR4_Pos )       // PD4            0: SEG7        1: ----        2: SCK11       3: ----        4: ----
                     | ( 0 << PD_PD_AFSR1_AFSR3_Pos )       // PD3            0: SEG8        1: RXD11       2: MISO11      3: ----        4: ----
                     | ( 0 << PD_PD_AFSR1_AFSR2_Pos )       // PD2            0: SEG9        1: TXD11       2: MOSI11      3: ----        4: ----
                     | ( 0 << PD_PD_AFSR1_AFSR1_Pos )       // PD1            0: SEG10       1: ----        2: ----        3: ----        4: ----
                     | ( 0 << PD_PD_AFSR1_AFSR0_Pos )       // PD0            0: SEG11       1: ----        2: ----        3: ----        4: ----
                     ;

      // configure PC0 ~ PC11 as a SEG23 ~ SEG12
      PC->MOD     =  0
                     | ( 2 << PC_PC_MOD_MODE11_Pos )        // PC11           0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE10_Pos )        // PC10           0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE9_Pos )         // PC9            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE8_Pos )         // PC8            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE7_Pos )         // PC7            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE6_Pos )         // PC6            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE5_Pos )         // PC5            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE4_Pos )         // PC4            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE3_Pos )         // PC3            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE2_Pos )         // PC2            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE1_Pos )         // PC1            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PC_PC_MOD_MODE0_Pos )         // PC0            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     ;
      PC->AFSR2   =  0
                     | ( 0 << PC_PC_AFSR2_AFSR11_Pos )      // PC11           0: SEG12       1: ----        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR2_AFSR10_Pos )      // PC10           0: SEG13       1: ----        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR2_AFSR9_Pos )       // PC9            0: SEG14       1: ----        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR2_AFSR8_Pos )       // PC8            0: SEG15       1: ----        2: ----        3: ----        4: ----
                     ;
      PC->AFSR1   =  0
                     | ( 0 << PC_PC_AFSR1_AFSR7_Pos )       // PC7            0: SEG16       1: ----        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR1_AFSR6_Pos )       // PC6            0: SEG17       1: ----        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR1_AFSR5_Pos )       // PC5            0: SEG18       1: ----        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR1_AFSR4_Pos )       // PC4            0: SEG19       1: ----        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR1_AFSR3_Pos )       // PC3            0: SEG20       1: EC21        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR1_AFSR2_Pos )       // PC2            0: SEG21       1: EC20        2: ----        3: ----        4: ----
                     | ( 0 << PC_PC_AFSR1_AFSR1_Pos )       // PC1            0: SEG22       1: T21OUT      2: T21CAP      3: ----        4: ----
                     | ( 0 << PC_PC_AFSR1_AFSR0_Pos )       // PC0            0: SEG23       1: T20OUT      2: T20CAP      3: ----        4: ----
                     ;

      // configure PB8 ~ PB15 as a SEG31 ~ SEG24
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 15, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 15, 0 );       // PB15           0: SEG24       1: ----        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 14, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 14, 0 );       // PB14           0: SEG25       1: ----        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 13, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 13, 0 );       // PB13           0: SEG26       1: ----        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 12, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 12, 0 );       // PB12           0: SEG27       1: ----        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 11, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 11, 0 );       // PB11           0: SEG28       1: T15OUT      2: T15CAP      3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 10, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 10, 0 );       // PB10           0: SEG29       1: ----        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 9, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 9, 0 );        // PB9            0: SEG30       1: EC15        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 8, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 8, 0 );        // PB8            0: SEG31       1: ----        2: ----        3: ----        4: ----

      // configure PB0 ~ PB7 as a SEG39 ~ SEG32 (except for PB4 ~ PB5)
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 7, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 7, 0 );        // PB7            0: SEG32       1: RXD1        2: ----        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 6, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 6, 0 );        // PB6            0: SEG33       1: TXD1        2: ----        3: ----        4: ----
      // HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 5, ALTERN_FUNC );
      // HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 5, 0 );        // PB5   SWDIO    0: SEG34       1: ----        2: SWDIO       3: ----        4: ----
      // HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 4, ALTERN_FUNC );
      // HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 4, 0 );        // PB4   SWCLK    0: SEG35       1: ----        2: SWCLK       3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 3, 0 );        // PB3            0: SEG36       1: BOOT        2: SS10        3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 2, 0 );        // PB2            0: SEG37       1: ----        2: SCK10       3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 1, 0 );        // PB1            0: SEG38       1: RXD10       2: MISO10      3: ----        4: ----
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 0, 0 );        // PB0            0: SEG39       1: TXD10       2: MOSI10      3: ----        4: ----
   }
}

static void l_init( void )
{
   int      i;

   for( i = 0; i < LCDBufSize; i++ )
   {
      lcdfbuf[i] = 0x00;
   }
}

static void l_disp( unsigned int disp )
{
   uint8_t     i;

   switch( disp % 4 )
   {
      case 0:
         for( i = 0; i < 10; i++ )
         {
            lcdfbuf[5 + i] = xLCD_BUF[( disp % 3 ) * 10 + i];
         }
         break;
      case 1:
         for( i = 0; i < 10; i++ )
         {
            lcdfbuf[5 + i] = 0xAA;;
         }
         break;
      case 2:
         for( i = 0; i < 10; i++ )
         {
            lcdfbuf[5 + i] = 0x55;
         }
         break;
      case 3:
         l_init();
         break;
      default:
         break;
   }
}

void t_lcd_8D3B( void )
{
   uint8_t     count = 0;

   DI();
   lcd_setting();
   l_init();
   EI();


   LCDON();

   while( 1 )
   {
      l_disp( count++ );

      msec = 500;
      while( msec );
   }
}

