/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void TIMER10_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER10_Configure( void );
void TIMER1n_MatchInterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER1n_MatchInterrupt Example \n\r"
                        "  - MCU:        A31R713 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: TIMER10 \n\r"
                        "GPIO Output Pin:   PE8 ~ PE11 \n\r"
                        "Timer Output Pin:  PE6(T10OUT) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER10_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER10_IRQHandler_IT( void )
{
   uint32_t          status;
   static uint32_t   count = 0;
   static uint8_t    alternate = 1;    // toggle flag

   // get interrupt flag
   status = T1nMaInt_GetFg( TIMER10 );

   // if Timer/Counter 1n Match Interrupt
   if( status == 1 )
   {
      // clear interrupt flag
      T1nMaInt_ClrFg( TIMER10 );

      // indicate
      if( ( count++ % 1000 ) == 0 )
      {
         // toggle PE8 ~ PE11
         if( alternate == 1 )
         {
            // clear flag
            alternate = 0;

            // output high
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 8 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 9 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 10 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 11 ) );
         }
         else
         {
            // set flag
            alternate = 1;

            // output low
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 8 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 9 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 10 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 11 ) );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PE8 ~ PE11 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PE8 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 8, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 8, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 8 ) );

      // configure PE9 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 9, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 9, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 9 ) );

      // configure PE10 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 10, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 10, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 10 ) );

      // configure PE11 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 11, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 11, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 11 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER10_Configure( void )
{
   TIMER1n_PERIODICCFG_Type      TIMER1n_Config;

   // TIMER10 configuration
   {
      // timer output pin configuration
      {
         // print
         _DBG( "PE6 was configured as a T10OUT. \n\r\n\r" );

         // configure PE6 as a T10OUT
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 6, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 6, AFSRx_AF1 );
      }

      // timer periodic mode configuration
      {
         TIMER1n_Config.CkSel = TIMER1n_PCLK;               // PCLK = 40MHz
         TIMER1n_Config.Prescaler = 40;                     // fTIMER = PCLK / 40 = 1MHz
         TIMER1n_Config.ADR = ( 1000 );                     // Period  = ADR / fTIMER = 1000us
         TIMER1n_Config.StartLevel = TIMER1n_START_LOW;
         TIMER1n_Config.BDR = 0;                            // Duty(B) = BDR / fTIMER =    0us  // dummy parameter
         TIMER1n_Config.ECE = TIMER1n_FALLING_EGDE;         // dummy parameter

         HAL_TIMER1n_Init( ( TIMER1n_Type* )TIMER10, TIMER1n_PERIODIC_MODE, &TIMER1n_Config );
         HAL_TIMER1n_ConfigInterrupt( ( TIMER1n_Type* )TIMER10, TIMER1n_INTCFG_MIE, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( TIMER10_IRQn, 3 );
      NVIC_EnableIRQ( TIMER10_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_TIMER10 );

      // enable timer (clear & start counter)
      HAL_TIMER1n_Cmd( ( TIMER1n_Type* )TIMER10, ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER1n_MatchInterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER1n_MatchInterruptRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER10_Configure */
   TIMER10_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* TIMER1n_MatchInterruptRun */
   TIMER1n_MatchInterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

