/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void TIMER20_Configure( void );
void TIMER2n_PWMRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER2n_PWM Example \n\r"
                        "  - MCU:        A31R713 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: TIMER20 \n\r"
                        "Timer Output Pin:  PC0(T20OUT) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER20_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER20_Configure( void )
{
   TIMER2n_PWMCFG_Type     TIMER2n_Config;

   // TIMER20 configuration
   {
      // timer output pin configuration
      {
         // print
         _DBG( "PC0 was configured as a T20OUT. \n\r\n\r" );

         // configure PC0 as a T20OUT
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 0, AFSRx_AF1 );
      }

      // timer pwm mode configuration
      {
         TIMER2n_Config.CkSel = TIMER2n_INTERNAL_CLK;       // PCLK = 40MHz
         TIMER2n_Config.Prescaler = 40;                     // fTIMER = PCLK / 40 = 1MHz
         TIMER2n_Config.ADR = ( 1000 );                     // Period  = ADR / fTIMER = 1000us
         TIMER2n_Config.BDR = ( 400 );                      // Duty(B) = BDR / fTIMER =  400us
         TIMER2n_Config.StartLevel = TIMER2n_START_HIGH;
         TIMER2n_Config.TCLK_SEL = TIMER2n_PCLK_CLK;
         TIMER2n_Config.CapEdge = TIMER2n_FALLING_EGDE;     // dummy parameter
         TIMER2n_Config.ExtCkEdge = TIMER2n_FALLING_EGDE;   // dummy parameter

         HAL_TIMER2n_Init( ( TIMER2n_Type* )TIMER20, TIMER2n_PWM_MODE, &TIMER2n_Config );
      }
   }

   // enable timer (clear & start counter)
   HAL_TIMER2n_Cmd( ( TIMER2n_Type* )TIMER20, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER2n_PWMRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER2n_PWMRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* TIMER20_Configure */
   TIMER20_Configure();

   /* TIMER2n_PWMRun */
   TIMER2n_PWMRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

