/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void SPI_SS_control( USART1n_Type* USART1x, char ch );
void USART10_IRQHandler_IT( void );
void USART11_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void SPI10_Configure( void );
void SPI11_Configure( void );
void USART1n_SPI_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "USART1n_SPI_Interrupt Example \n\r"
                        "  - MCU:        A31R713 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:   SPI10 Master \n\r"
                        "Counterpart:         SPI11 Slave \n\r"
                        "Master Tx Data:      'A' ~ 'Z' \n\r"
                        "Slave Tx Data:       'a' ~ 'z' \n\r"
                        "SPI Master MOSI Pin: PB0(MOSI10)      \n\r"
                        "SPI Master MISO Pin: PB1(MISO10)      \n\r"
                        "SPI Master SCLK Pin: PB2(SCK10)       \n\r"
                        "SPI Master nSS Pin:  PB3(Output Mode) \n\r"
                        "SPI Slave MOSI Pin:  PD2(MOSI11) \n\r"
                        "SPI Slave MISO Pin:  PD3(MISO11) \n\r"
                        "SPI Slave SCLK Pin:  PD4(SCK11)  \n\r"
                        "SPI Slave nSS Pin:   PD5(SS11)   \n\r"
                        "Prerequisite:        Connect PB0(MOSI10)      and PD2(MOSI11) with a jumper. \n\r"
                        "                     Connect PB1(MISO10)      and PD3(MISO11) with a jumper. \n\r"
                        "                     Connect PB2(SCK10)       and PD4(SCK11)  with a jumper. \n\r"
                        "                     Connect PB3(Output Mode) and PD5(SS11)   with a jumper. \n\r"
                        "************************************************\n\r\n\r";

volatile uint8_t     FlagMasterRXC = 0;
volatile uint8_t     mrxch;
volatile uint8_t     FlagSlaveRXC = 0;
volatile uint8_t     srxch;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI_SS_control
 * @param[in]     USART1x
 * @param[in]     ch
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI_SS_control( USART1n_Type* USART1x, char ch )
{
#if 1    // supported
   if( USART1x == ( USART1n_Type* )USART10 )
   {
      if( ch == 0 )
      {
         PB->OUTDR_b.OUTDR3 = 0;    // PB3   BOOT     0: SEG36       1: BOOT        2: SS10        3: ----        4: ----
      }
      else
      {
         PB->OUTDR_b.OUTDR3 = 1;
      }
   }
#endif

#if 1    // supported
   if( USART1x == ( USART1n_Type* )USART11 )
   {
      if( ch == 0 )
      {
         PD->OUTDR_b.OUTDR5 = 0;    // PD5            0: SEG6        1: ----        2: SS11        3: ----        4: ----
      }
      else
      {
         PD->OUTDR_b.OUTDR5 = 1;
      }
   }
#endif

#if 0    // not supported
   if( USART1x == ( USART1n_Type* )USART12 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR15 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR15 = 1;
      }
   }
#endif

#if 0    // not supported
   if( USART1x == ( USART1n_Type* )USART13 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR11 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR11 = 1;
      }
   }
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART10_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART10_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   // get interrupt flag
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART10 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // if Receive Complete Interrupt
   if( USART10->CR1_b.RXCIEn == 1 && ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      // set flag
      FlagMasterRXC = 1;

      // get byte
      mrxch = HAL_USART_ReceiveByte( ( USART1n_Type* )USART10 );
   }

   // if Transmit Complete Interrupt
   if( USART10->CR1_b.TXCIEn == 1 && ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      // clear interrupt flag
      HAL_USART_ClearStatus( ( USART1n_Type* )USART10, USART1n_STATUS_TXC );

      // nss = 1
      SPI_SS_control( ( USART1n_Type* )USART10, 1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART11_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART11_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   // get interrupt flag
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART11 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // if Receive Complete Interrupt
   if( USART11->CR1_b.RXCIEn == 1 && ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      // set flag
      FlagSlaveRXC = 1;

      // get byte
      srxch = HAL_USART_ReceiveByte( ( USART1n_Type* )USART11 );
   }

   // if Transmit Complete Interrupt
   if( USART11->CR1_b.TXCIEn == 1 && ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      // clear interrupt flag
      HAL_USART_ClearStatus( ( USART1n_Type* )USART11, USART1n_STATUS_TXC );

      // nss = 1
      SPI_SS_control( ( USART1n_Type* )USART11, 1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // SPI10 configuration
   {
      // spi MOSI/MISO/SCLK/nSS pin configuration
      {
         // print
         _DBG( "PB0 was configured as a MOSI10. \n\r" );
         _DBG( "PB1 was configured as a MISO10. \n\r" );
         _DBG( "PB2 was configured as a SCK10. \n\r" );
         _DBG( "PB3 was configured as a Output Mode. \n\r\n\r" );

         // configure PB0 as a MOSI10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 0, AFSRx_AF2 );   // PB0            0: SEG39       1: TXD10       2: MOSI10      3: ----        4: ----

         // configure PB1 as a MISO10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 1, AFSRx_AF2 );   // PB1            0: SEG38       1: RXD10       2: MISO10      3: ----        4: ----

         // configure PB2 as a SCK10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 2, AFSRx_AF2 );   // PB2            0: SEG37       1: ----        2: SCK10       3: ----        4: ----

         // configure PB3 as a Output Mode
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT ); // PB3   BOOT     0: SEG36       1: BOOT        2: SS10        3: ----        4: ----
      }

      // spi master mode configuration
      {
         // get default USART1n_Config for spi: 38400-8-N-1
         HAL_USART_SPI_Mode_Config( &USART1n_Config );

         // init usart1n
         HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

         // enable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, ENABLE );

         // nss = 1
         SPI_SS_control( ( USART1n_Type* )USART10, 1 );

         // enable interrupt
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART10, USART1n_INTCFG_TXC, ENABLE );
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART10, USART1n_INTCFG_RXC, ENABLE );

         // enable usart1n
         HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( USART10_IRQn, 3 );
      NVIC_EnableIRQ( USART10_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_USART10 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI11_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI11_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // SPI11 configuration
   {
      // spi MOSI/MISO/SCLK/nSS pin configuration
      {
         // print
         _DBG( "PD2 was configured as a MOSI11. \n\r" );
         _DBG( "PD3 was configured as a MISO11. \n\r" );
         _DBG( "PD4 was configured as a SCK11. \n\r" );
         _DBG( "PD5 was configured as a SS11. \n\r\n\r" );

         // configure PD2 as a MOSI11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );      // PE11           0: ----        1: ----        2: ----        3: (TXD11)     4: (MOSI11)
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, AFSRx_AF2 );   // PD2            0: SEG9        1: TXD11       2: MOSI11      3: ----        4: ----

         // configure PD3 as a MISO11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 3, ALTERN_FUNC );      // PE10           0: ----        1: ----        2: ----        3: (RXD11)     4: (MISO11)
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 3, AFSRx_AF2 );   // PD3            0: SEG8        1: RXD11       2: MISO11      3: ----        4: ----

         // configure PD4 as a SCK11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, ALTERN_FUNC );      // PE9            0: ----        1: ----        2: ----        3: (SCK11)     4: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 4, AFSRx_AF2 );   // PD4            0: SEG7        1: ----        2: SCK11       3: ----        4: ----

         // configure PD5 as a SS11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 5, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 5, AFSRx_AF2 );   // PD5            0: SEG6        1: ----        2: SS11        3: ----        4: ----
      }

      // spi slave mode configuration
      {
         // get default USART1n_Config for spi: 38400-8-N-1
         HAL_USART_SPI_Mode_Config( &USART1n_Config );

         // init usart1n
         HAL_USART_Init( ( USART1n_Type* )USART11, &USART1n_Config );

         // disable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_MASTER, DISABLE );

         // enable slave nss pin function
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_USTSSEN, ENABLE );

         // enable interrupt
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART11, USART1n_INTCFG_TXC, ENABLE );
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART11, USART1n_INTCFG_RXC, ENABLE );

         // enable usart1n
         HAL_USART_Enable( ( USART1n_Type* )USART11, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( USART11_IRQn, 3 );
      NVIC_EnableIRQ( USART11_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_USART11 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_SPI_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_SPI_InterruptRun( void )
{
   uint8_t     mtxch = 'A';   // master tx character
   uint8_t     stxch = 'a';   // slave tx character

   /* Infinite loop */
   while( 1 )
   {
      // slave tx: 'a' ~ 'z'
      HAL_USART_Transmit( ( USART1n_Type* )USART11, &stxch, 1, NONE_BLOCKING );

      // master tx: 'A' ~ 'Z'
      SPI_SS_control( ( USART1n_Type* )USART10, 0 );
      HAL_USART_Transmit( ( USART1n_Type* )USART10, &mtxch, 1, NONE_BLOCKING );

      // master rx
      while( 1 )
      {
         if( FlagMasterRXC == 1 )
         {
            // clear flag
            FlagMasterRXC = 0;

            // exit
            break;
         }
      }

      // slave rx
      while( 1 )
      {
         if( FlagSlaveRXC == 1 )
         {
            // clear flag
            FlagSlaveRXC = 0;

            // exit
            break;
         }
      }

      // print
      _DBG( "Master Tx Data = " );
      _DBC( mtxch );
      _DBG( "   " );
      _DBG( "Slave Tx Data  = " );
      _DBC( stxch );
      _DBG( "\n\r" );
      _DBG( "Slave Rx Data  = " );
      _DBC( srxch );
      _DBG( "   " );
      _DBG( "Master Rx Data = " );
      _DBC( mrxch );
      _DBG( "\n\r\n\r" );

      // mtxch++
      if( mtxch == 'Z' )
      {
         mtxch = 'A';
      }
      else
      {
         mtxch++;
      }

      // stxch++
      if( stxch == 'z' )
      {
         stxch = 'a';
      }
      else
      {
         stxch++;
      }

      // delay
      delay();
      delay();
      delay();
      delay();
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* SPI10_Configure */
   SPI10_Configure();

   /* SPI11_Configure */
   SPI11_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* USART1n_SPI_InterruptRun */
   USART1n_SPI_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

