/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void UART10_Configure( void );
void USART1n_UART_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "USART1n_UART_Polling Example \n\r"
                        "  - MCU:        A31R713 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: UART10 \n\r"
                        "Counterpart:       A external device like a PC \n\r"
                        "Tx Data:           Any character \n\r"
                        "UART TXD Pin:      PB0(TXD10) \n\r"
                        "UART RXD Pin:      PB1(RXD10) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // UART10 configuration
   {
      // uart TXD/RXD pin configuration
      {
         // print
         _DBG( "PB0 was configured as a TXD10. \n\r" );
         _DBG( "PB1 was configured as a RXD10. \n\r\n\r" );

         // configure PB0 as a TXD10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 0, AFSRx_AF1 );    // PB0            0: SEG39       1: TXD10       2: MOSI10      3: ----        4: ----

         // configure PB1 as a RXD10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 1, AFSRx_AF1 );    // PB1            0: SEG38       1: RXD10       2: MISO10      3: ----        4: ----
      }

      // uart configuration
      {
         // get default configuration for uart: 38400-8-N-1
         HAL_USART_UART_Mode_Config( &USART1n_Config );

         // init USART1n
         HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

         // enable USART1n
         HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_UART_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_UART_PollingRun( void )
{
   uint8_t     buf[100];      // tx/rx buffer
   uint32_t    len;

   /* Infinite loop */
   while( 1 )
   {
      // receive data from PC
      len = HAL_USART_Receive( ( USART1n_Type* )USART10, buf, sizeof( buf ), BLOCKING );

      // send the received data to PC
      if( len > 0 )
      {
         HAL_USART_Transmit( ( USART1n_Type* )USART10, buf, len, BLOCKING );
         if( buf[len - 1] == '\r' )
         {
            buf[0] = '\n';
            HAL_USART_Transmit( ( USART1n_Type* )USART10, buf, 1, BLOCKING );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* UART10_Configure */
   UART10_Configure();

   /* USART1n_UART_PollingRun */
   USART1n_UART_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

