/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void USART10_IRQHandler_IT( void );
void USART11_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void USRT10_Configure( void );
void USRT11_Configure( void );
void USART1n_USRT_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "USART1n_USRT_Interrupt Example \n\r"
                        "  - MCU:        A31R713 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:     USRT10 Master \n\r"
                        "Counterpart:           USRT11 Slave \n\r"
                        "Master Tx Data:        'A' ~ 'Z' \n\r"
                        "Slave Tx Data:         'a' ~ 'z' \n\r"
                        "USRT Master TXD Pin:   PB0(TXD10) \n\r"
                        "USRT Master RXD Pin:   PB1(RXD10) \n\r"
                        "USRT Master SCK Pin:   PB2(SCK10) \n\r"
                        "USRT Slave TXD Pin:    PD2(TXD11) \n\r"
                        "USRT Slave RXD Pin:    PD3(RXD11) \n\r"
                        "USRT Slave SCK Pin:    PD4(SCK11) \n\r"
                        "Prerequisite:          Connect PB0(TXD10) and PD3(RXD11) with a jumper. \n\r"
                        "                       Connect PB1(RXD10) and PD2(TXD11) with a jumper. \n\r"
                        "                       Connect PB2(SCK10) and PD4(SCK11) with a jumper. \n\r"
                        "************************************************\n\r\n\r";

volatile uint8_t     FlagMasterRXC = 0;
volatile uint8_t     mrxch;
volatile uint8_t     FlagSlaveRXC = 0;
volatile uint8_t     srxch;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART10_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART10_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   // get interrupt flag
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART10 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // if Receive Complete Interrupt
   if( USART10->CR1_b.RXCIEn == 1 && ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      // set flag
      FlagMasterRXC = 1;

      // get byte
      mrxch = HAL_USART_ReceiveByte( ( USART1n_Type* )USART10 );
   }

   // if Transmit Complete Interrupt
   if( USART10->CR1_b.TXCIEn == 1 && ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      // clear interrupt flag
      HAL_USART_ClearStatus( ( USART1n_Type* )USART10, USART1n_STATUS_TXC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART11_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART11_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   // get interrupt flag
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART11 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // if Receive Complete Interrupt
   if( USART11->CR1_b.RXCIEn == 1 && ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      // set flag
      FlagSlaveRXC = 1;

      // get byte
      srxch = HAL_USART_ReceiveByte( ( USART1n_Type* )USART11 );
   }

   // if Transmit Complete Interrupt
   if( USART11->CR1_b.TXCIEn == 1 && ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      // clear interrupt flag
      HAL_USART_ClearStatus( ( USART1n_Type* )USART11, USART1n_STATUS_TXC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         USRT10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USRT10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // USRT10 configuration
   {
      // usrt TXD/RXD/SCK pin configuration
      {
         // print
         _DBG( "PB0 was configured as a TXD10. \n\r" );
         _DBG( "PB1 was configured as a RXD10. \n\r" );
         _DBG( "PB2 was configured as a SCK10. \n\r\n\r" );

         // configure PB0 as a TXD10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 0, AFSRx_AF1 );   // PB0            0: SEG39       1: TXD10       2: MOSI10      3: ----        4: ----

         // configure PB1 as a RXD10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 1, AFSRx_AF1 );   // PB1            0: SEG38       1: RXD10       2: MISO10      3: ----        4: ----

         // configure PB2 as a SCK10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 2, AFSRx_AF2 );   // PB2            0: SEG37       1: ----        2: SCK10       3: ----        4: ----
      }

      // usrt master mode configuration
      {
         // get default USART1n_Config for usrt: 38400-8-N-1
         HAL_USART_USRT_Mode_Config( &USART1n_Config );

         // init usart1n
         HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

         // enable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, ENABLE );

         // select clock waveform
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_DISSCK, DISABLE );

         // enable interrupt
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART10, USART1n_INTCFG_TXC, ENABLE );
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART10, USART1n_INTCFG_RXC, ENABLE );

         // enable usart1n
         HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( USART10_IRQn, 3 );
      NVIC_EnableIRQ( USART10_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_USART10 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USRT11_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USRT11_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // USRT11 configuration
   {
      // usrt TXD/RXD/SCK pin configuration
      {
         // print
         _DBG( "PD2 was configured as a TXD11. \n\r" );
         _DBG( "PD3 was configured as a RXD11. \n\r" );
         _DBG( "PD4 was configured as a SCK11. \n\r\n\r" );

         // configure PD2 as a TXD11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );      // PE11           0: ----        1: ----        2: ----        3: (TXD11)     4: (MOSI11)
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, AFSRx_AF1 );   // PD2            0: SEG9        1: TXD11       2: MOSI11      3: ----        4: ----

         // configure PD3 as a RXD11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 3, ALTERN_FUNC );      // PE11           0: ----        1: ----        2: ----        3: (TXD11)     4: (MOSI11)
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 3, AFSRx_AF1 );   // PD3            0: SEG8        1: RXD11       2: MISO11      3: ----        4: ----

         // configure PD4 as a SCK11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, ALTERN_FUNC );      // PE9            0: ----        1: ----        2: ----        3: (SCK11)     4: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 4, AFSRx_AF2 );   // PD4            0: SEG7        1: ----        2: SCK11       3: ----        4: ----
      }

      // usrt slave mode configuration
      {
         // get default USART1n_Config for usrt: 38400-8-N-1
         HAL_USART_USRT_Mode_Config( &USART1n_Config );

         // init usart1n
         HAL_USART_Init( ( USART1n_Type* )USART11, &USART1n_Config );

         // disable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_MASTER, DISABLE );

         // enable interrupt
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART11, USART1n_INTCFG_TXC, ENABLE );
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART11, USART1n_INTCFG_RXC, ENABLE );

         // enable usart1n
         HAL_USART_Enable( ( USART1n_Type* )USART11, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( USART11_IRQn, 3 );
      NVIC_EnableIRQ( USART11_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_USART11 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_USRT_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_USRT_InterruptRun( void )
{
   uint8_t     mtxch = 'A';   // master tx character
   uint8_t     stxch = 'a';   // slave tx character

   /* Infinite loop */
   while( 1 )
   {
      // slave tx: 'a' ~ 'z'
      HAL_USART_Transmit( ( USART1n_Type* )USART11, &stxch, 1, NONE_BLOCKING );

      // master tx: 'A' ~ 'Z'
      HAL_USART_Transmit( ( USART1n_Type* )USART10, &mtxch, 1, NONE_BLOCKING );

      // master rx
      while( 1 )
      {
         if( FlagMasterRXC == 1 )
         {
            // clear flag
            FlagMasterRXC = 0;

            // exit
            break;
         }
      }

      // slave rx
      while( 1 )
      {
         if( FlagSlaveRXC == 1 )
         {
            // clear flag
            FlagSlaveRXC = 0;

            // exit
            break;
         }
      }

      // print
      _DBG( "Master Tx Data = " );
      _DBC( mtxch );
      _DBG( "   " );
      _DBG( "Slave Tx Data  = " );
      _DBC( stxch );
      _DBG( "\n\r" );
      _DBG( "Slave Rx Data  = " );
      _DBC( srxch );
      _DBG( "   " );
      _DBG( "Master Rx Data = " );
      _DBC( mrxch );
      _DBG( "\n\r\n\r" );

      // mtxch++
      if( mtxch == 'Z' )
      {
         mtxch = 'A';
      }
      else
      {
         mtxch++;
      }

      // stxch++
      if( stxch == 'z' )
      {
         stxch = 'a';
      }
      else
      {
         stxch++;
      }

      // delay
      delay();
      delay();
      delay();
      delay();
      delay();
   }
}

// Configuration Sequence: slave, master
//  ٲ Master Rx Data ó .
/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* USRT11_Configure */
   USRT11_Configure();

   /* USRT10_Configure */
   USRT10_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* USART1n_USRT_InterruptRun */
   USART1n_USRT_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

